@@ -18,7 +18,7 @@
 #' @importFrom grDevices dev.cur dev.off dev.set pdf
 #' @importFrom stats na.omit quantile
 #' @importFrom utils capture.output
-#' @useDynLib primerTree
+#' @useDynLib primerTree rawStreamToDNAbin
 NULL
 
 #' PrimerTree results for the mammalian 16S primers
@@ -117,7 +117,7 @@ plot.primerTree = function(x, ranks=NULL, main=NULL, ...){
 #'  num_aligns=1000, total_primer_specificity_mismatch=3)
 #' }
 search_primer_pair = function(forward, reverse, name=NULL, num_aligns=500,
-    num_permutations=25, simplify=TRUE, clustal_options=list(), distance_options=list(model='raw'),
+    num_permutations=25, simplify=TRUE, clustal_options=list(), distance_options=list(model="N", pairwise.deletion=T),
     ..., .parallel=FALSE, .progress='none'){
 
   #HACK, primerTree is an environment rather than a list so we can treat it as
@@ -151,6 +151,8 @@ search_primer_pair = function(forward, reverse, name=NULL, num_aligns=500,
       primer_search$BLAST_result =
         filter_duplicates(ldply(primer_search$response, parse_primer_hits, .parallel=.parallel))
 
+      primer_search$BLAST_result$gi <- as.character(primer_search$BLAST_result$gi)
+      
       message(nrow(primer_search$BLAST_result), ' BLAST alignments parsed in ', seconds_elapsed_text(start_time))
 
       start_time = now()
@@ -178,7 +180,7 @@ search_primer_pair = function(forward, reverse, name=NULL, num_aligns=500,
               ' length:', ncol(primer_search$alignment))
 
       start_time = now()
-      primer_search$distances = do.call(dist.dna, c(list(primer_search$alignment, distance_options)))
+      primer_search$distances = do.call(dist.dna, c(list(primer_search$alignment), distance_options))
       message('pairwise DNA distances calculated in ',
               seconds_elapsed_text(start_time))
 
@@ -247,10 +249,13 @@ summary.primerTree <- function(object, ..., probs=c(0, .05, .5, .95, 1), ranks =
   res[['lengths']] = t(data.frame('Sequence lengths'=labeled_quantile(laply(object$sequence, length), sprintf('%.0f%%', probs*100), probs=probs), check.names=F))
   print(res[['lengths']])
 
-  res[['distances']] = t(data.frame('Pairwise differences'=labeled_quantile(object$distance, sprintf('%.0f%%', probs*100), probs=probs), check.names=F))
+  res[['distances']] = t(data.frame('Pairwise differences'=labeled_quantile(object$distances, sprintf('%.0f%%', probs*100), probs=probs), check.names=F))
   cat('\n')
   print(res[['distances']])
 
+  res[['rankDistances']] = calc_rank_dist_ave(object, common_ranks)
+  print(res[['rankDistances']])
+
   res[['ranks']] = laply(object$taxonomy[common_ranks], function(x) length(unique(x)))
   cat('\n', 'Unique taxa out of ', length(object$sequence), ' sequences\n', sep='')
   names(res[['ranks']]) = ranks
@@ -264,3 +269,124 @@ labeled_quantile = function(x, labels, ...){
   names(res) = labels
   res
 }
+
+#' Summarize pairwise differences.
+
+#' @param x a primerTree object
+#' @param ranks ranks to show unique counts for, defaults to the common ranks
+#' @return returns a data frame of results
+#' @details
+#' The purpose of this function is to calculate the average number
+#' of nucleotide differences between species within each taxa of given taxonomic
+#' level.
+#'
+#' For example, at the genus level, the function calculates the average number
+#' of nucleotide differences between all species within each genus and reports
+#' the mean of those values.
+#'
+#' There are several key assumptions and calculations made in this
+#' function.
+#'
+#' First, the function randomly selects one sequence from each species
+#' in the primerTree results. This is to keep any one species (e.g.
+#' human, cow, etc.) with many hits from skewing the results.
+#'
+#' Second, for each taxonomic level tested, the function divides the
+#' sequences by each taxon at that level and calculates the mean
+#' number of nucleotide differences within that taxa, then returns the
+#' mean of those values.
+#'
+#' Third, when calculating the average distance, any taxa for which
+#' there is only one species is omitted, as the number of nucleotide
+#' differences will always be 0.
+#'
+#' @examples
+#' \dontrun{
+#' calc_rank_dist_ave(mammals_16S)
+#'
+#' calc_rank_dist_ave(bryophytes_trnL)
+#'
+#' # Note that the differences between the results from these two primers
+#' # the mean nucleotide differences is much higher for the mammal primers
+#' # than the byrophyte primers. This suggests that the mammal primers have
+#' # better resolution to distinguish individual species.
+#' }
+#' @importFrom reshape2 melt
+#' @export
+
+# using tree data format info from http://www.phytools.org/eqg/Exercise_3.2/
+calc_rank_dist_ave <- function(x, ranks = common_ranks) {
+  used_ranks <- grep("species", ranks, invert = T, value = T)
+  rank_dist_mean <- data.frame(matrix(nrow = 1, ncol = 0))
+
+  # Raw taxonomy data
+  taxa <- as.data.frame(x$taxonomy)
+
+  # Randomize the order of the taxa data frame
+  taxa <- taxa[sample(nrow(taxa)), ]
+  rownames(taxa) <- taxa$gi
+
+  # Pick random example per species and add back in the taxa info
+  unique_factors <- as.data.frame(unique(taxa$species))
+  colnames(unique_factors) <- "species"
+  unique_factors <- join(unique_factors, taxa, type = "left", match = "first", by = "species")
+
+  # Get sequences for randomly selected species
+  seqs <- x$sequence
+  seqs <- seqs[names(seqs) %in% unique_factors$gi]
+
+  # Align and calculate pairwise distances and convert dists to dataframe
+  align <- clustalo(seqs)
+  dists <- as.data.frame(as.matrix(dist.dna(align, model = "N", pairwise.deletion = T)))
+  dists$gi <- row.names(dists)
+
+  # Melt the dists dataframe so I can drop any distance that isn't within the (rank)
+  melted <- melt(dists, id = "gi", variable.name = "gi2", value.name = "dist")
+
+  for(rank in used_ranks) {
+
+    # Gather only the needed taxa data
+    unique_factors_sub <- unique_factors[ , colnames(unique_factors) %in% c("gi", "species", rank)]
+
+    # Drop any row in (rank) where there is only one species represented
+    # Any instance of this leads to a distance within that rank of 0, skewing the results downward
+    counts <- as.data.frame(table(unique_factors_sub[[rank]]))
+    colnames(counts) <- c(rank, "count")
+    unique_factors_sub <- join(unique_factors_sub, counts, by = rank)
+    unique_factors_sub <- unique_factors_sub[unique_factors_sub$count > 1, ]
+
+    # Pull the nucleotide distance data in
+    # Replace the rank1 gi with the rank1 taxa
+    melted_sub <- join(melted, unique_factors_sub,by = "gi")
+    melted_sub$rank1 <- as.factor(melted_sub[[rank]])
+
+    # Drop all columns except the three needed so the next join doesn't get messed up
+    melted_sub <- melted_sub[, colnames(melted_sub) %in% c("gi2", "dist", "rank1", "species")]
+    colnames(melted_sub)[1] <- "gi"
+
+    # Replace the rank2 gi with the rank2 taxa
+    melted_sub <- join(melted_sub, unique_factors_sub, by = "gi")
+    melted_sub$rank2 <- as.factor(melted_sub[[rank]])
+
+    # Drop all columns except the three needed
+    melted_sub <- melted_sub[ , colnames(melted_sub) %in% c("rank2", "dist", "rank1", "species")]
+
+    # Drop all rows with missing information
+    melted_sub <- na.omit(melted_sub)
+
+    # We only want distances within a taxa, so drop all comparisons between taxa
+    # We also want to drop any comparisons of a species to itself, which will have dist == 0
+    melted_sub <- melted_sub[melted_sub$rank1 == melted_sub$rank2 & melted_sub$species != melted_sub$species.1, ]
+
+    # Calculate the mean distance for each taxa compared
+    #   We calculate each separately to avoid any one taxa with lots of hits (like human seqs)
+    #   from skewing the mean
+    melted_sub$group <- paste(melted_sub$rank1, melted_sub$rank2)
+
+    # Plug the means into the storage dataframe
+    rank_dist_mean[[rank]] <- mean(melted_sub$dist)
+  }
+  message("\nAverage number of nucleotide differences between sequences within a given taxonomic group")
+  message("See function description for further details")
+  rank_dist_mean
+}
